/*
 * Component: sr
 *
 * Description:
 *  Header file containing the definition of CodeManager object
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */
#ifndef CODE_TABLE_H_INCLUDED
#define CODE_TABLE_H_INCLUDED

#include <stdio.h>
#include <string.h>

#include <dcmtk/ofstd/oflist.h>
#include <dcmtk/dcmsr/dsrcodvl.h>

class CodingElement {
	private:

		char * m_value;
		char * m_meaning;

	public:
		CodingElement( const char * value, const char * meaning );
		~CodingElement();

		inline const char * getValue() const {
			return m_value;
		}

		inline const char * getMeaning() const {
			return m_meaning;
		}

		/**
		 * Compares a givem code value with own code value
		 *
		 * @param value   Code Value to compare
		 *
		 * @return an integer  less  than,  equal to,  or  greater  than zero if own value
		 *         is found respectively to be less than, to match, or be greater than the given value
		 */
		inline int compareTo( const char * value ) const {
			return strcmp( m_value, value );
		}
};

class CodeTable {
	private:
		char * m_designator;
		OFList< CodingElement * > m_codes;

		void loadCodeTable( const char * filename );

	public:
		CodeTable( const char * designator, const char * filename );

		~CodeTable();

		inline const char * getDesignator() const {
			return m_designator;
		}

		const char * getMeaning( const char * value );

		void listCodes( FILE * out );
};

class CodeManager {
	private:
		OFList< CodeTable * > m_tables;
		char * m_directory;

	public:
		CodeManager( const char * directory = NULL );
		~CodeManager();

		const char * getCodeMeaning( const char * value, const char * designator );

		void listDesignatorCodes( FILE * out, const char * designator );
		void listAll( FILE * out );

		DSRCodedEntryValue getCodedEntry( const char * codeValue, const char * designator );
};

#endif								  // CODE_TABLE_H_INCLUDED
