/*
 * Component: sr
 *
 * Description:
 *  Header file containing the definition of
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */
#ifndef DOSE_INFO_H_INCLUDED
#define DOSE_INFO_H_INCLUDED

#include <stdio.h>
#include <string.h>
#include <dcmtk/ofstd/ofstring.h>
#include <dcmtk/ofstd/oflist.h>
#include <dcmtk/ofstd/ofstd.h>
#include <dcmtk/dcmsr/dsrdoc.h>
#include <dcmtk/dcmsr/dsrstrvl.h>

#include "CodeTable.h"

class DoseInfoCollector;

class DoseInfo {
	friend class DoseInfoCollector;
	private:
		OFString m_series;
		OFString m_targetRegion;
		OFString m_phantomType;
		OFString m_CTDIvol;
		OFString m_DLP;

	protected:
		DoseInfo() {
			m_series = "undefined";
			m_targetRegion = "-";
			m_phantomType = "-";
			m_CTDIvol = "-";
			m_DLP = "-";
		}

		void setCTDIvol( const OFString & ctdIvol ) {
			m_CTDIvol = ctdIvol;
		}

		void setDLP( const OFString & dlp ) {
			m_DLP = dlp;
		}

		void setPhantomType( const OFString & phantomType ) {
			m_phantomType = phantomType;
		}

		void setSeries( const OFString & series ) {
			m_series = series;
		}

		void setTargetRegion( const OFString & targetRegion ) {
			m_targetRegion = targetRegion;
		}


	public:

		const OFString & getCTDIvol() const {
			return m_CTDIvol;
		}

		const OFString & getDLP() const {
			return m_DLP;
		}

		const OFString & getPhantomType() const {
			return m_phantomType;
		}

		const OFString & getSeries() const {
			return m_series;
		}

		const OFString & getTargetRegion() const {
			return m_targetRegion;
		}

		DoseInfo( const DoseInfo & orig ) {
			m_series = orig.m_series;
			m_targetRegion = orig.m_targetRegion;
			m_phantomType = orig.m_phantomType;
			m_CTDIvol = orig.m_CTDIvol;
			m_DLP = orig.m_DLP;
		}

		DoseInfo & operator=( const DoseInfo & orig ) {
			m_series = orig.m_series;
			m_targetRegion = orig.m_targetRegion;
			m_phantomType = orig.m_phantomType;
			m_CTDIvol = orig.m_CTDIvol;
			m_DLP = orig.m_DLP;

			return *this;
		}
};

/**
 * Usage:
 *
 * <code>
 * DoseInfoCollector dc( study );
 * try {
 *   dc.collect();
 * }
 * catch ( IMException & ex ) {
 *   // handle errors
 * }
 * dc.getDoseInfo();
 * </code>
 */
class DoseInfoCollector:public DSRTypes {
	private:
		OFString m_study;
		DSRDocumentTree * m_tree;
		OFList< DoseInfo > m_infos;
		DoseInfo * m_actInfo;
		OFString m_series;

		CodeManager * m_codeManager;
		DSRCodedEntryValue m_CT_Acquisition;
		DSRCodedEntryValue m_CTDIvol;
		DSRCodedEntryValue m_PhantomType;
		DSRCodedEntryValue m_DLP;
		DSRCodedEntryValue m_TargetRegion;
		DSRCodedEntryValue m_SeriesUID;

		void searchSubTree( OFBool isContinuous );
		void searchTree( );

	public:
		DoseInfoCollector( const OFString & study );
		~DoseInfoCollector();

		void collect();

		const OFList< DoseInfo > & getDoseInfo() const {
			return m_infos;
		}
};


#endif								  // DOSE_INFO_H_INCLUDED
