/**
 * Component: 
 *
 * Description:
 * 
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */
#ifndef IMATTACHMENT_H_
#define IMATTACHMENT_H_

#include "IMDocument.h"
#include "OrderDataFiller.h"


/**
 * Base class for IM attachments (scanned documents)
 */
class IMAttachment : public IMDocument {

	/** used in createNew() only: true indicates to create a regular study object instead of attachment */
	OFBool m_isRegularObject;
	/** used in createNew() only: object to get demographic information from */
	OrderDataFiller * m_odf;

	public:

		/**
		 * Constructor for creating a new empty IMAttachment object.
		 *
		 * @param odf       Order Data Filler object to get demographic information from for creating new attachments
		 * @param isRegular save object as regular image/object rather than as attachment
		 */
		IMAttachment( OrderDataFiller * odf, OFBool isRegular );

		/**
		 * Constructor for creating a new IMAttachment object for an existing document.
		 *
		 * @param filename   File name of the existing document
		 */
		IMAttachment( const char * filename );

		/**
		 * Destructor for destroying an IMAttachment object.
		 */
		virtual ~IMAttachment();

		/**
		 * Creates new ImageMedical Attachment for the given study.
		 * Creates SOP Instance UID and file name for the new document.
		 *
		 * @param studyIUID   Study Instance UID of the study the document will belong to
		 *
		 * @throw IMException if an error occures
		 */
		virtual void createNew( const char * studyIUID );

		/**
		 * Sets series description for the attachment
		 *
		 * @param description New series description
		 *
		 * @return EC_Normal on success or appropriate error condition
		 */
		inline OFCondition setSeriesDescription( const OFString & description ) {
			if ( m_dataset == NULL )
				return EC_IllegalCall;
			return m_dataset->putAndInsertOFStringArray( DCM_SeriesDescription, description );
		}

		/**
		 * Sets series UID for the attachment
		 *
		 * @param seriesUID  New series UID
		 *
		 * @return EC_Normal on success or appropriate error condition
		 */
		inline OFCondition setSeriesUID( const char * seriesUID ) {
			if ( m_dataset == NULL )
				return EC_IllegalCall;
			if ( seriesUID == NULL )
				return EC_IllegalParameter;
			return m_dataset->putAndInsertString( DCM_SeriesInstanceUID, seriesUID );
		}

		/**
		 * Sets series number for the attachment
		 *
		 * @param seriesNumber  New series number
		 *
		 * @return EC_Normal on success or appropriate error condition
		 */
		inline OFCondition setSeriesNumber( Sint32 seriesNumber ) {
			if ( m_dataset == NULL )
				return EC_IllegalCall;
			char sn[32];
			snprintf( sn, 32, "%ld", seriesNumber );
			return m_dataset->putAndInsertString( DCM_SeriesNumber, sn );
		}

		/**
		 * Sets instance number for the attachment
		 *
		 * @param instanceNumber  New instance number
		 *
		 * @return EC_Normal on success or appropriate error condition
		 */
		inline OFCondition setInstanceNumber( Sint32 instanceNumber ) {
			if ( m_dataset == NULL )
				return EC_IllegalCall;
			char in[32];
			snprintf( in, 32, "%ld", instanceNumber );
			return m_dataset->putAndInsertString( DCM_InstanceNumber, in );
		}

		/**
		 * Sets modality for the object
		 *
		 * @param modality  new modality
		 *
		 * @return EC_Normal on success or appropriate error condition
		 */
		inline OFCondition setModality( const char * modality ) {
			if ( m_dataset == NULL )
				return EC_IllegalCall;
			if ( modality == NULL )
				return EC_IllegalParameter;
			return m_dataset->putAndInsertString( DCM_Modality, modality );

		}

		/**
		 * Sets study date/time for the object
		 *
		 * @param studyDate  new study date
		 * @param studyTime  new study time
		 *
		 * @return EC_Normal on success or appropriate error condition
		 */
		inline OFCondition setStudyDateTime( const char * studyDate, const char * studyTime ) {
			if ( m_dataset == NULL )
				return EC_IllegalCall;
			if ( studyDate == NULL )
				return EC_IllegalParameter;

			OFCondition cond = m_dataset->putAndInsertString( DCM_StudyDate, studyDate );
			if ( cond.good() && studyTime != NULL ) {
				OFCondition cond = m_dataset->putAndInsertString( DCM_StudyTime, studyTime );
			}

			return cond;
		}


		/**
		 * Sets attachment type for the attachment
		 *
		 * @param attachmentType   New attachment type
		 *
		 * @return EC_Normal on success or appropriate error condition
		 */
		OFCondition setAttachmentType( const char * attachmentType );


		/**
		 * Calculate series number for a series to be added to the study
		 *
		 * @param study  Study Instance UID of the study the series to be added to
		 *
		 * @return  series number to be used
		 */
		static int calcSeriesNum( const char * study );
};

#endif /* IMATTACHMENT_H_ */
