/*
 * Component: sr
 *
 * Description:
 *  Header file containing the definition of IMDocument object
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */
#ifndef IM_IMAGE_H_INCLUDED
#define IM_IMAGE_H_INCLUDED

#include "IMAttachment.h"

/**
 * Creates and manipulates Secondary Capture Image attachments
 */
class IMImage : public IMAttachment {
	protected:

		/**
		 * Sets Image Medical specific tags and values in the Document
		 */
		virtual void setIMTags();


	public:

		/**
		 * Constructor for creating a new empty IMImage object.
		 *
		 * @param odf        Order Data Filler object to get demographic information from for creating new attachments
		 * @param isRegular  save object as regular image rather than as attachment
		 */
		IMImage( OrderDataFiller * odf, OFBool isRegular );

		/**
		 * Constructor for creating a new IMImage object for an existing document.
		 *
		 * @param filename   File name of the existing document
		 */
		IMImage( const char * filename );

		/**
		 * Destructor for destroying an IMImage object.
		 */
		virtual ~IMImage();

		/**
		 * @return  identifier of document type (SC image)
		 */
		virtual E_IMDocType getIdent() const {
			return IMDocImg;
		}

		/**
		 * Creates new ImageMedical Secondary Captured Image for the given study.
		 * Creates SOP Instance UID and file name for the new document.
		 * Pixel Data information will not be created
		 *
		 * @param studyIUID   Study Instance UID of the study the document will belong to
		 *
		 * @throw IMException if an error occures
		 */
		virtual void createNew( const char * studyIUID );

		/**
		 * Inserts an RGB image to a newly created Secondary Captured Image
		 *
		 * @param pixelDataFile  path to file storing the rgb jpeg image
		 * @param rows           rows in the image
		 * @param columns        columns in the image
		 *
		 * @return EC_Normal on success or appropriate error condition
		 */
		OFCondition insertRgbImage( const char * pixelDataFile, Uint16 rows, Uint16 columns );
};

#endif								  // IM_IMAGE_H_INCLUDED
