/*
 * Component: sr
 *
 * Description:
 *  Header file for Image Medical Order Manager implementation.
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */
#ifndef IM_ORDER_MANAGER_H_INCLUDED
#define IM_ORDER_MANAGER_H_INCLUDED

#include <dcmtk/ofstd/ofstd.h>

#include "IMLogger.h"
#include "StoreState.h"
#include "dbim.h"

#include "IMPbRDocument.h"
#include "CodeTable.h"
#include "EventList.h"
#include "LogList.h"

#include "dtconv.h"

class IMOrderManager {
	private:

		char * m_study;
		IMPbRDocument * m_pbrDocument;
		CodeManager * m_codeManager;
		EventList m_eventList;
		LogList m_logList;
		OFBool m_notEditable;
		OFBool m_notDeletable;
		OFBool m_removeStoreStateRec;
		StoreState m_storeState;
		IMLogger m_logger;

		OFBool isValidStatus( const char * status, int & code );

		OFBool setStatus( DcmDataset * pbrDataset, DcmElement * status, OFBool addEvents = OFTrue );
		OFBool setStatus(	DcmDataset * pbrDataset, const char * orderst, OFBool addEvents = OFFalse );

		void checkAndCreateStorestateRec();

	public:

		IMOrderManager( const char * study, const char * codeTableDir = NULL );
		~IMOrderManager();

		void load();
		void save();

		void cancel( OFBool isCompleted, const char * userid, const char * deviceid );

		void loadPbR( DB_Query & db );

		void processModificationDataset( pbtz::OffsetAdjustedDataset & ofDataset, OFBool createOnly = OFFalse );

		void getLastModifiedDateTime( const char *& lmdate, const char *& lmtime );

		void log( const char * userid, const char * device );

		void copyElement( DcmItem* initem, DcmItem* outitem, DcmTag tag );

		DcmItem * findScheduledProcedureStep( const char* stepUID );
		DcmItem * findOurPerformedProcedureItem( const char* ourPPSID );

		void MPPSCreated( const char* stepUID, DcmDataset* indataset );
		bool MPPSCompleted( const char* stepUID, const char* PPSID, DcmDataset* indataset );
		bool MPPSDiscontinued( const char* stepUID, const char* PPSID, DcmDataset* indataset );

		IMPbRDocument * getOrCreatePbRDocument();

		inline IMPbRDocument * getPbRDocument() {
			return m_pbrDocument;
		}
		inline EventList & getEventList() {
			return m_eventList;
		}
		inline char * getStudy() {
			return m_study;
		}

		inline void setNotEditable( OFBool flag ) {
			m_notEditable = flag;
		}
		inline void setNotDeletable( OFBool flag ) {
			m_notDeletable = flag;
		}
};

#endif								  // IM_ORDER_MANAGER_H_INCLUDED
