/**
 * Component: 
 *
 * Description:
 * 
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */
#ifndef IMPDFDOCUMENT_H_
#define IMPDFDOCUMENT_H_

#include <dcmtk/dcmdata/dccodec.h>

#include "IMAttachment.h"

/**
 * Creates and manipulates Encapsulated PDF Document attachments
 */
class IMPDFDocument : public IMAttachment {
	public:

		/**
		 * Constructor for creating a new empty IMPDFDocument object.
		 *
		 * @param odf        Order Data Filler object to get demographic information from for creating new attachments
		 * @param isRegular  save object as regular image rather than as attachment
		 */
		IMPDFDocument( OrderDataFiller * odf, OFBool isRegular );

		/**
		 * Constructor for creating a new IMPDFDocument object for an existing document.
		 *
		 * @param filename   File name of the existing document
		 */
		IMPDFDocument( const char * filename );

		/**
		 * Destructor for destroying an IMPDFDocument object.
		 */
		virtual ~IMPDFDocument();


		/**
		 * @return  identifier of document type (PDF document)
		 */
		virtual E_IMDocType getIdent() const {
			return IMDocPdf;
		}

		/**
		 * Creates new ImageMedical Encapsulated PDF Document for the given study.
		 * Creates SOP Instance UID and file name for the new document.
		 * PDF content will not be created
		 *
		 * @param studyIUID   Study Instance UID of the study the document will belong to
		 *
		 * @throw IMException if an error occures
		 */
		virtual void createNew(const char * studyIUID);

		/**
		 * Inserts a PDF content to a newly created Encapsulated PDF Document
		 *
		 * @param pixelDataFile  path to file PDF content
		 *
		 * @return EC_Normal on success or appropriate error condition
		 */
		OFCondition insertPdfContent( const char * filename );

		/**
		 * Sets 'Burned in annotation' flag for the document
		 *
		 * @param hasBurnedInAnnotation   indicates whether the document contains burned in annotations
		 *
		 * @return EC_Normal on success or appropriate error condition
		 */
		inline OFCondition setBurnedInAnnotation( bool hasBurnedInAnnotation ) {
			if ( m_dataset == NULL )
				return EC_IllegalCall;
			return m_dataset->putAndInsertString( DCM_BurnedInAnnotation, hasBurnedInAnnotation ? "YES" : "NO" );
		}

		/**
		 * Sets document title
		 *
		 * @param title  document title
		 *
		 * @return EC_Normal on success or appropriate error condition
		 */
		inline OFCondition setTitle( const char * title ) {
			if ( m_dataset == NULL )
				return EC_IllegalCall;
			if ( title == NULL )
				return EC_IllegalParameter;
			return m_dataset->putAndInsertString( DCM_DocumentTitle, title );
		}

		/**
		 * Sets document title as concept name code sequence
		 *
		 * @param codingScheme   Coding Scheme Designator
		 * @param codeValue      Code Value
		 * @param codeMeaning    Code Meaning
		 *
		 * @return EC_Normal on success or appropriate error condition
		 */
		inline OFCondition setConceptName( const char * codingScheme, const char * codeValue, const char * codeMeaning ) {
			if ( m_dataset == NULL )
				return EC_IllegalCall;
			if ( codingScheme == NULL || codeValue == NULL || codeMeaning == NULL )
				return EC_IllegalParameter;
			return DcmCodec::insertCodeSequence( m_dataset, DCM_ConceptNameCodeSequence, codingScheme, codeValue, codeMeaning );
		}
};

#endif /* IMPDFDOCUMENT_H_ */
