/*
 * Component: sr
 *
 * Description:
 *  Header file containing the definition of IMReportManager class.
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */
#ifndef IM_REPORT_MANAGER_H_INCLUDED
#define IM_REPORT_MANAGER_H_INCLUDED

#include <dcmtk/ofstd/ofstd.h>

#include "Lock.h"
#include "dbim.h"

#include "IMSRDocument.h"
#include "IMPbDDocument.h"
#include "IMPbRDocument.h"
#include "SRDisplay.h"
#include "CodeTable.h"
#include "LogList.h"
#include "EventList.h"

class IMReportManager {
	private:

		char * m_study;
		OFList< IMSRDocument * > m_SRList;
		OFList< SRDisplay * > m_externalList;
		IMPbDDocument * m_pbdDocument;
		IMPbRDocument * m_pbrDocument;
		CodeManager * m_codeManager;
		OFBool m_converted;
		OFBool m_bMainDocumentCreated;
		EventList m_eventList;
		LogList m_logList;OFBool m_conversionForceFlag;
		Lock * m_lock;

		void addDocumentToList( IMSRDocument * srDocument );

		void calcMainST( int & mainStatus, int curStatus );
		void getIDsFromSeq( DcmSequenceOfItems * seq, char *& uid, char *& id );

		void loadPbR( DB_Query & db, const char * timeZone = NULL );
		void loadDictation( DB_Query & db, const char * timeZone = NULL );
		void loadReports( DB_Query & db, const char * timeZone = NULL );
		void loadExternalReports( DB_Query & db );

//	void writeReports();

		void applyDemographyChanges( pbtz::OffsetAdjustedDataset & ofDataset, DcmItem * item );
		void applyDictationChanges( pbtz::OffsetAdjustedDataset & ofDataset, DcmSequenceOfItems * sequence );
		void applyInterpretationItemChanges( pbtz::OffsetAdjustedDataset & ofDataset, DcmItem * item );
		void applyInterpretationChanges( pbtz::OffsetAdjustedDataset & ofDataset, DcmSequenceOfItems * sequence );
		void applyResultsItemChanges( pbtz::OffsetAdjustedDataset & ofDataset, DcmItem * item );
		void applyResultsChanges( pbtz::OffsetAdjustedDataset & ofDataset, DcmSequenceOfItems * sequence );

		OFIterator< IMSRDocument * > getBeginIterator();
		OFIterator< IMSRDocument * > getEndIterator();

	public:

		IMReportManager( const char * study, const char * codeTableDir = NULL );
		~IMReportManager();

		void load( const char * timeZone = NULL );
		void save();

		inline void setConversionForceFlag( OFBool forceFlag ) {
			m_conversionForceFlag = forceFlag;
		}
		void convertPbR( char * pbrFileName, char * pbdFileName );
		void convertPbR( char * fileName );

//	void modifyReports(char * fileName);

		IMSRDocument * createNewSRDocument( E_IMSRDocType docType, DcmDataset * demographyDataset = NULL );
		IMSRDocument * getMergeSRDocument( E_IMSRDocType docType );

		IMSRDocument * getMainDocument();
		IMSRDocument * getSRDocument( const char * srUID );
		IMSRDocument * getSRDocumentByName( const char * fileName );
		IMSRDocument * getSRDocumentForResults( const char * uid, const char * id );
		IMSRDocument * getSRDocumentForInterpretation( const char * uid, const char * id );

		void processModificationDataset( pbtz::OffsetAdjustedDataset & ofDataset );

		void dump( FILE * output = stdout, OFBool createImaFiles = OFFalse, OFBool withSound = OFFalse,
				OFBool embeddedSound = OFFalse, const char * wavePrefix = NULL );
		void getLastModifiedDateTime( const char *& lmdate, const char *& lmtime );

		inline OFBool isConvertedReport() {
			return m_converted;
		}

		void log( const char * userid, const char * device );

		inline EventList & getEventList() {
			return m_eventList;
		}

		inline LogList & getLogList() {
			return m_logList;
		}

		inline char * getStudy() {
			return m_study;
		}

		int getMainStatus();
};

#endif								  // IM_REPORT_MANAGER_H_INCLUDED
