/*
 * Component: sr
 *
 * Description:
 *  Header file containing the definition of LogList.
 *  Also contains the definition of abstract class LogEntry
 *  and its descendants ReportLogEntry and StudyLogEntry.
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */
#ifndef LOG_LIST_H_INCLUDED
#define LOG_LIST_H_INCLUDED

#include <dcmtk/ofstd/ofstd.h>
#include <dcmtk/ofstd/oflist.h>
#include <dcmtk/dcmdata/dcdatset.h>

class LogEntry {
 protected:
	char * m_logAction;

 public:
	LogEntry(const char * logAction);
	virtual ~LogEntry();

	virtual void log(const char * user, const char * study, const char * name) = 0;
};


class ReportLogEntry : public LogEntry {
 private:
	char m_intID[32];
	EVENT_CODE m_eventCode;
	int m_status;

	const char * eventToString();

 public:
	ReportLogEntry(const char * intID, EVENT_CODE eventCode, int status);
	virtual ~ReportLogEntry();

	virtual void log(const char * user, const char * study, const char * name);
};


class StudyLogEntry : public LogEntry {
 private:

	const char * m_modality;
	const char * m_name;
	const char * m_status;

 public:
	StudyLogEntry(const char * logAction, DcmDataset * dataset);

	virtual ~StudyLogEntry();

	virtual void log(const char * user, const char * study, const char * name);
};



class LogList {
 private:

	OFList < LogEntry * > m_list;
	char * m_logAction;
	char * m_logTag;

 public:
	LogList(const char * logTag, const char * logAction);
	~LogList();

	void log(DcmDataset * dataset, const char * userid, const char * device, const char * studyIUID);

	void addLogEntry(LogEntry * entry);
};

#endif								  // LOG_LIST_H_INCLUDED
