/**
 * Component: 
 *
 * Description:
 * 
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */

#ifndef SANITIZER_H_
#define SANITIZER_H_

#include <dcmtk/ofstd/ofstring.h>
#include <dcmtk/ofstd/oflist.h>
#include <dcmtk/dcmdata/dctk.h>

#include "IMPbRDocument.h"

/**
 * Helper class to create sanitization item for copy/shortcut PbR
 */
class Sanitizer {

	/** Study Instance UID of the original study */
	const char * m_study;

	/** Filename of mapping file for sanitization */
	const char * m_mapFile;

	/** Sanitized data */
	DcmDataset * m_sanitizedData;

	/** Original data */
	DcmDataset * m_originalData;

	/** Full path to study directory */
	OFString m_studyDir;

	/** Name of the PbR file (relative to study directory) */
	OFString m_pbrFileName;

	/** Name of the Min SOP I file (relative to study directory) */
	OFString m_minsopiFileName;

	/** List of the sanitized user defined custom fields */
	OFList<OFString> m_sanitizedFields;

public:

	/**
	 * Constructs a new <code>Sanitizer</code> object.
	 *
	 * @param study    Study to be sanitized/edited
	 * @param mapFile  File describing sanitization/editing mapping
	 */
	Sanitizer( const char * study, const char * mapFile );

	/**
	 * Destructs this <code>Sanitizer</code> object
	 */
	virtual ~Sanitizer();

	/**
	 * @return true if sanitized data is available
	 */
	inline bool hasSanitizedData() const {
		return m_sanitizedData != NULL;
	}

	/**
	 * Get original study data.
	 * @return original study data
	 */
	inline DcmDataset * getOriginalData() {
		return m_originalData;
	}

	/**
	 * Get sanitized data.
	 * Indented to be used for logging purposes only.
	 * @return sanitized data
	 */
	inline DcmDataset * getSanitizedData() {
		return m_sanitizedData;
	}


	/**
	 * Returns full path to study directory
	 * @return study directory
	 */
	inline const OFString & getStudyDirectory() const {
		return m_studyDir;
	}

	/**
	 * Return PbR file name (relative to study directory)
	 * @return PbR file name
	 */
	inline const OFString & getPbRFileName() const {
		return m_pbrFileName;
	}

	/**
	 * Return Min SOP I file name (relative to study directory)
	 * @return Min SOP I file name
	 */
	inline const OFString & getMinSOPIFileName() const {
		return m_minsopiFileName;
	}


	/**
	 * Finds study directory, PbR file and MIN SOP I file for the study.
	 */
	bool findStudyFiles();

	/**
	 * Collects original data for sanitization from MIN SOP I UID and PbR object of source study
	 */
	void collectOriginalData();

	/**
	 * Does the sanitization
	 */
	bool sanitize( bool needStudyFiles = OFFalse );


	/**
	 * Applies sanitized data to pbrDataset.
	 */
	bool applyTo( IMPbRDocument & pbrDocument );
};


#endif /* SANITIZER_H_ */
