/*
 * Component: iface
 *
 * Description:
 *      Header file for message transfer configuration file parsing routines
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */
#ifndef __DEVICE_CONFIG_H_INCLUDED__
#define __DEVICE_CONFIG_H_INCLUDED__

#include <string.h>

#include "IMConfig.h"

#include "EventList.h"

typedef enum {
	PROTOCOL_NULL = 0,
	PROTOCOL_HL7 = 1,
	PROTOCOL_DIMSEN = 2,
	PROTOCOL_STORAGE = 3
} MESSAGE_PROTOCOL;

// TODO: SendToDevice is not used from C/C++ code
// and does not fit to new event handling
// Need to be checked if DeviceEntry/DeviceConfig to be used at all or to be reorganized


//class SendToDevice {
//private:
//	char * m_deviceID;
//	unsigned long m_events;
//
//public:
//	SendToDevice();
//	~SendToDevice();
//
//	OFBool init(const char * entry);
//
//	inline const char * getDeviceID() const {
//		return m_deviceID;
//	}
//
//	inline unsigned long getEvents() const {
//		return m_events;
//	}
//};


class DeviceConfig;

class DeviceEntry : public IMConfigEntry {
		friend class DeviceConfig;
	private:

		char * m_ipAddress;
		int m_port;
		char * m_aeTitle;
		char * m_vmdFile;
		char * m_sendTo;
		//	SendToDevice      * m_sendToArray;
		//	int                 m_sizeOfSendTo;
		MESSAGE_PROTOCOL m_protocol;

		char *skipPipe( char * str );

	public:

		inline const char * getIPAddress() const {
			return m_ipAddress;
		}

		inline const char * getAETitle() const {
			return m_aeTitle;
		}

		inline const char * getVmdFile() const {
			return m_vmdFile;
		}

		inline int getPort() const {
			return m_port;
		}

		inline MESSAGE_PROTOCOL getProtocol() const {
			return m_protocol;
		}

		//	inline void getSendToArray(SendToDevice *& sendTo, int & sizeOfSendTo) {
		//		sendTo = m_sendToArray; sizeOfSendTo = m_sizeOfSendTo;
		//	}

	protected:

		virtual void init();

		DeviceEntry();
		DeviceEntry( const char * id, const char * ipAddress, const char * port, const char * aetitle, const char * vmdFile,
				const char * sendTo, const char * protocol );

		virtual ~DeviceEntry();
};

class DeviceConfig : public IMConfig {
	private:

		virtual IMConfigEntry * processLine( char * line );
		virtual IMConfigEntry * createHeadEntry();

	public:
		DeviceConfig( const char * configFileName );
		DeviceConfig();
		virtual ~DeviceConfig();

		const char * getDimsenDeviceID( const char * ip, const char * aetitle );
		const char * getHL7DeviceID( const char * ip );
};

#endif // __DEVICE_CONFIG_H_INCLUDED__
