/*
 * Component: iface
 *
 * Description:
 *   Header file for internal interface for editing studies
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */
#ifndef __EDIT_IFACE_H_INCLUDED__
#define __EDIT_IFACE_H_INCLUDED__

#include <dcmtk/ofstd/oflist.h>
#include <dcmtk/dcmdata/dctk.h>

#include "IMDBLogger.h"
#include "dbim.h"

#include "ordermatchingcfg.h"


/**
 * Type to control how studies to be edited
 */
typedef enum {
	/** read default setting from environment/ config file ?? */
	EDIT_MODE_DEFAULT,//!< EDIT_MODE_DEFAULT
	/** edit only the PbR file */
	EDIT_MODE_SOFT,   //!< EDIT_MODE_SOFT
	/** copy all changes from the PbR file into the image files */
	EDIT_MODE_HARD    //!< EDIT_MODE_HARD
} EDIT_MODE;


class MergeConfig {
	private:

		OFList< DcmTagKey > m_hardTags;
		OFList< DcmTagKey > m_softTags;

	public:

		MergeConfig();
		~MergeConfig();

		void readMergeConfig( DB_Query * db, const char *configFile = NULL );

		bool isHardTag( const DcmTagKey & key );
		bool isSoftTag( const DcmTagKey & key );
		bool isMergeTag( const DcmTagKey & key );

		void addSoftTag( const DcmTagKey & key );
		void addHardTag( const DcmTagKey & key );

		void clearHardTags();
		void clearSoftTags();
};

class IMEditIface {
	private:
		DB_Query m_db;
		MergeConfig m_mergeConfig;
		bool m_createLogEntry;
		IMDBLogger m_logger;
		EDIT_MODE m_editMode;

		/** True indicates to print generated order UID to standard output */
		bool m_printGeneratedUID;

		OrderMatchingConfig * m_omConfig;

		void editOrder( const char * study, DcmDataset & dataset, DcmDataset * mergeDataset, const char * userid,
				const char * device );
		void editStudy( const char * study, DcmDataset & dataset, DcmDataset * mergeDataset, const char * userid,
				const char * device, bool isOrder );

		void editPbR( const char * study, DcmDataset & dataset, DcmDataset * mergeDataset, const char * userid, const char * device, DcmDataset ** updatedPbR = NULL );
		void editImages( const char * study, const char * series, DcmDataset * pbrDataset );
		void editImages( const char * study );

		char * createLogMessage( DcmItem * dataset );
		char * removeWhiteSpacesForLog( char * str );
		void log( const char * study, const char * series, DcmItem * dataset, const char * userid, const char * device );

		static void applyModificationsOnImageFile( DcmDataset & pbrDataset, const char * fileName );

	public:

		IMEditIface( EDIT_MODE m_editMode = EDIT_MODE_SOFT, bool createLogEntry = true );
		~IMEditIface();

		void readMergeConfig( const char * configFileName = NULL );

		void edit( const char * study, DcmDataset & dataset, DcmDataset * mergeDataset, const char * userid, const char * device );
		void editSeries( const char * study, const char * series, DcmItem * changesItem, const char * userid, const char * device );
		void createOrder( DcmDataset & dataset, const char * userid, const char * device );

		OFString editFromRIS( DcmDataset * dataset, const char * user, const char * device, bool createNew, bool checkValidOrder, bool checkOwner );

		/**
		 * Edits a study using coercion rules
		 *
		 * @param study     study to be edited
		 * @param mapFile   file with coercion rules to be used
		 * @param userid    ID of the user initiating study edit
		 */
		void coerceStudy( const char * study, const char * mapFile, const char * userid );

		static EDIT_MODE getDefaultEditMode();

		/**
		 * Sets whether newly generated order UID should be printed to the standard output or not.
		 * If the value is set, the UID will be printed in the following format (followed by a new line):
		 * ORDER UID: <code>UID</code>
		 *
		 * Only takes affect when a new order to be created.
		 *
		 * @param newValue   true indicates to print generated order UID to the standard out.
		 */
		inline void setPrintGeneratedUID( bool newValue ) {
			m_printGeneratedUID = newValue;
		}
};

#endif								  // __EDIT_IFACE_H_INCLUDED__
