/*
 * Component: iface
 *
 * Description:
 *   Header file of internal interface for processing incoming orders
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */
#ifndef __ORDER_IFACE_H_INCLUDED__
#define __ORDER_IFACE_H_INCLUDED__

#include "dconfig.h"
#include "common.h"
#include "IMLogger.h"

#include "IMOrderManager.h"
#include "ordermatchingcfg.h"

class IMOrderIface {
	private:
		DeviceConfig * m_dcfg;
		IMLogger m_logger;
		OrderMatchingConfig * m_matchingConfig;
		OrderGroupingConfig * m_groupingConfig;

		void sendEvents( IMOrderManager & orderManager, const char * device );

	public:
		IMOrderIface();
		virtual ~IMOrderIface();

		int import( DcmDataset * orderDataset, const char * configFileName, const char * userid, const char * device,
				bool checkValidOrder = true );
		int import( DcmDataset * orderDataset, DcmDataset * mergeDataset, const char * configFileName, const char * userid,
				const char * device, bool checkValidOrder = true );

		inline const char * getDimsenDeviceID( const char * ip, const char * aetitle ) {
			return m_dcfg == NULL ? NULL : m_dcfg->getDimsenDeviceID( ip, aetitle );
		}

		inline const char * getHL7DeviceID( const char * ip ) {
			return m_dcfg == NULL ? NULL : m_dcfg->getHL7DeviceID( ip );
		}

		int processDimsenOrder( DcmDataset * dataset, const char * ipAddress, const char * aeTitle );

		int MPPSCreated( const char * studyUID, const char * stepUID, DcmDataset * indataset );
		bool MPPSCompleted( const char * studyUID, const char * stepUID, const char * PPSID, DcmDataset * indataset );
		bool MPPSDiscontinued( const char * studyUID, const char * stepUID, const char * PPSID, DcmDataset * indataset );

};

#endif // __ORDER_TASK_H_INCLUDED__
