/*
 * presentationstate.h
 *
 *  Created on: Aug 28, 2013
 *      Author: tnemeth
 */

#ifndef PRESENTATIONSTATE_H_
#define PRESENTATIONSTATE_H_

namespace impsconverter {

enum SIZE_MODE {
	SIZE_MODE_FIT,
	SIZE_MODE_ZOOM,
	SIZE_MODE_FIT_ACTUAL,
	SIZE_MODE_ZOOM_ACTUAL
};

enum CLIP_BOX_MODE {
	CLIP_BOX_NORMAL,
	CLIP_BOX_90,
	CLIP_BOX_180,
	CLIP_BOX_270
};

class Annotation;

class PresentationState {
	friend class Annotation;
	private:
		/** Content of the new Presentation State object */
		DcmFileFormat m_dcmff;

		/** File name for the Presentation State object */
		OFString m_fileName;

		DcmItem * m_displayedAreaSelectionItem;

		SIZE_MODE m_sizeMode;
		CLIP_BOX_MODE m_clipBoxMode;

		DcmItem * m_annotations;

		bool m_hasAnnotationLayer;

		/** Serial number of the presentation within the original EP presentation state */
		int m_presentationNumber;

		/** true indicates to print debug information */
		bool m_debug;

		/**
		 * Remove unnecessary elements
		 */
		void clean();

	protected:

		inline bool isDebug() const {
			return m_debug;
		}

	public:
		PresentationState( DcmDataset * dataset, const OFString & baseUID, int presentationNumber, bool debug = false );
		~PresentationState();

		/**
		 * Saves the Presentation State object.
		 * Prefix will be prepended to the auto-generated file name or the one set by setFileName().
		 * The intention is that prefix is a directory name including a trailing directory separator ('/').
		 * @param prefix prefix to be prepended to file name
		 *
		 * @return true if and only if the Presentation State object was successfully saved.
		 */
		bool save( const OFString & prefix );

		/**
		 * Use this filename to save the Presentation State object.
		 * If not set, then the fileName will be generated from the SOP Class and SOP Instance.
		 *
		 * @param fileName new file name for the Presentation State object
		 */
		inline void setFileName( const OFString & fileName ) {
			m_fileName = fileName;
		}

		inline OFCondition createNewCompoundGraphicItem( DcmItem *& item ) {
			return m_annotations->findOrCreateSequenceItem( DCM_CompoundGraphicSequence, item, -2 );
		}

		inline OFCondition createNewTextObjectItem( DcmItem *& item ) {
			return m_annotations->findOrCreateSequenceItem( DCM_TextObjectSequence, item, -2 );
		}

		inline OFCondition createNewGraphicObjectItem( DcmItem *& item ) {
			return m_annotations->findOrCreateSequenceItem( DCM_GraphicObjectSequence, item, -2 );
		}


		bool setImageReferences( const OFString & sopClass, const OFString & sopInstance, const OFString & frame, const OFString & series );
		bool setOrientation( int rotation, bool isFlipped );
		bool setSizeMode( SIZE_MODE sizeMode );
		bool setPixelSpacing( double pixelX, double pixelY );
		bool setZoom( double zoomX, double zoomY );
		bool setClip( int posX, int posY, int sizeX, int sizeY );
		bool setWindowLevel( int window, int level, bool isInverted );

		bool checkAndCreateAnnotationLayer();
};


}


#endif /* PRESENTATIONSTATE_H_ */
