/*
 * psconverter.h
 *
 *  Created on: Aug 28, 2013
 *      Author: tnemeth
 */

#ifndef PSCONVERTER_H_
#define PSCONVERTER_H_

#include "presentationstate.h"
#include "annotation.h"
#include "ruler.h"
#include "oval.h"

namespace impsconverter {

class PSConverter {
private:
		/** Data set of the original Presentation State object */
		DcmDataset * m_dataset;
		/** Reference Series Sequence of the original object */
		DcmSequenceOfItems * m_referencedSeriesSequence;

		/** Counter for referenced images in PBFigure */
		int m_counter;
		/** SOP Instance UID of the original Presentation State object */
		OFString m_instanceUID;

		/** Current presentation state being processed */
		PresentationState * m_currentPresentation;

		/** true indicates to write debug messages */
		bool m_debug;

		OFString m_savePrefix;

		void genUid( OFString & uid );

		bool parseFigure( const char * figure );
		bool parsePluginState( const Uint8 * pluginState, Uint32 length );

		bool processIDLine( OFString & line );
		bool processLightLine( OFString & line );
		bool processSizedLine( OFString & line );
		bool processPixelLine( OFString & line );
		bool processClipLine( OFString & line );

		bool findSOPClass( const OFString & fname, OFString & sopInstance, OFString & sopClass, OFString & series );

public:
		PSConverter( DcmDataset * dataset, const OFString & savePrefix, bool debug = false );
		~PSConverter();

		bool convert();
};


/**
 * Checks whether provided SOP class is a Presentation State SOP class supported for conversion
 *
 * @param sopClass   SOP Class UID to check
 *
 * @return true if the SOP class is a Presentation State SOP class supported for conversion
 */
bool isPSSopClass( const char * sopClass );


/**
 * Checks whether an object file need to be converted or not
 *
 * @param dcmff      Content of the DICOM object
 * @param sopClass   SOP Class UID of the DICOM object ((optional, can be passed if already known)
 *
 * @return true if the object is a presentation state object and contains presentation in PB private elements
 */
bool needPSConversion( DcmFileFormat & dcmff, const char * sopClass = NULL );

/**
 * Deletes a directory recursively
 *
 * @param  dirPath   path to the directory to delete
 *
 * @return true if and only if the directory was deleted successfully
 */
bool deleteDirectoryRecursively( const OFString & dirPath );


/**
 * Adds full file path of the objects in a directory to a file list.
 *
 * @param dirPath   path to the directory to list
 * @param fileList  add file paths to this list
 *
 * @return true if and only if the directory was successfully listed
 */
bool addObjects( const OFString & dirPath, OFList<OFString> & fileList );

/**
 * Runs convPS application to convert a Presentation State object.
 * The generated objects will be stored in the provided output directory.
 *
 * @param objectFile   the Presentation State object file to convert
 * @param outputDir    output directory for the generated files
 *
 * @return true if and only if the Presentation State object file was successfully convert
 */
bool runConvertPSObject( const OFString & objectFile, const OFString & outputDir );

}

#endif /* PSCONVERTER_H_ */
