/*
 * Component: iface
 *
 * Description:
 *   Header file of internal interface for processing incoming reports
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */
#ifndef __REPORT_IFACE_H_INCLUDED__
#define __REPORT_IFACE_H_INCLUDED__

#include "imtask.h"
#include "dconfig.h"
#include "IMLogger.h"

typedef enum {
	PROCMODE_Normal = 0,
	PROCMODE_Merge = 1
} E_ReportProcessMode;

#include "IMReportManager.h"
#include "ordermatchingcfg.h"

class DimsenEventList;

class IMReportIface {
	private:
		DeviceConfig * m_dcfg;
		IMLogger m_logger;
		ReportMatchingConfig m_matchingCfg;

		int m_counter;

		int getCounter() {
			return ( m_counter & 0x40000000 ) ? ( m_counter = 0 ) : m_counter++;
		}

		bool studyExists( const OFString & study, bool & existsFlag, OFString & dbuid );

	public:
		IMReportIface();
		virtual ~IMReportIface();

		int import( DcmDataset * reportDataset, const char * configFileName, const char * userid, const char * device, E_ReportProcessMode procMode =
				PROCMODE_Normal );

		inline const char * getDimsenDeviceID( const char * ip, const char * aetitle ) {
			return m_dcfg == NULL ? NULL : m_dcfg->getDimsenDeviceID( ip, aetitle );
		}

		inline const char * getHL7DeviceID( const char * ip ) {
			return m_dcfg == NULL ? NULL : m_dcfg->getHL7DeviceID( ip );
		}

		int processDimsenReport( DcmDataset * dataset, const char * ipAddress, const char * aeTitle );
		int processHL7Report( DcmDataset * dataset, const char * deviceID, bool doMerge );
};

#endif // __REPORT_IFACE_H_INCLUDED__
